
#include "comb/necklace.h"

#include "jjassert.h"

#include "nextarg.h"
#include "fxttypes.h"  // ulong

#include "fxtio.h"

//#include "comb/is-setpart-rgs.h"

//% Generate all pre-necklaces, necklaces, and Lyndon words with a given number of colors.


// OEIS sequences:
//
// A062692 (pre-necklaces, 2 cols.)
// A000031 (necklaces, 2 cols.)
// A001037 (Lyndon words, 2 cols.)
//
// A114945 (pre-necklaces, 3 cols.)
// A001867 (necklaces, 3 cols.)
// A027376 (Lyndon words, 3 cols.)

// cf. http://oeis.org/wiki/Index_to_OEIS:_Section_Lu#Lyndon
// cf. http://oeis.org/wiki/Index_to_OEIS:_Section_Ne#necklaces

// Cf. comb/lyndon-words-demo.cc
// Cf. comb/lyndon-words-prefix-cond-demo.cc


#define PRX()  { if (x[k])  cout << x[k]; else cout << '.'; }

//#define TIMING  // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 4;
    NXARG(n, "length of strings >=1");
    ulong m = 3;
    NXARG(m, "number of different beads >=2");

    necklace P(n, m);

    const ulong *x = P.data();
    (void)x;

    ulong j = P.j_;  // ==1
    ulong pct = 0;  // count pre-necklaces (pre-prime strings)
    ulong ct = 0;   // count necklaces (prime strings)
    ulong lct = 0;  // count Lyndon words
    do
    {
#if 0  // OEIS sequences:
        { // no two adjacent elements identical:
            // A106365 (m=3), A106366 (m=4), A106367 (m=5), A106368 (m=6)
            if ( n>1 ) if ( x[0] == x[n-1] )  continue;
            bool q = true;
            for (ulong k=1; k<n; ++k)  if ( x[k]==x[k-1] )  { q=false; break; }
            if ( !q )  continue;
        }
#endif
#if 0  // OEIS sequences:
        if ( !is_setpart_rgs(x, n) )  continue;  // modulo color permutations
        // m = 3:
        // echo $(for n in $(seq 1 12); do ./bin $n 3; done | g yndon | cut -d= -f3 | cut -d\# -f1) | sed 's/ /, /g;'
        // necklaces: 1, 2, 4, 10, 24, 66, 172, 474, 1289, 3560, 9820, 27327 = A000000
        // Lyndon words: 1, 1, 3, 8, 23, 61, 171, 464, 1285, 3535, 9819, 27253 = A000000
#endif
        ++pct;
        ct += P.is_necklace();
        lct += P.is_lyndon_word();

#if defined TIMING
#else  // TIMING

        cout << setw(4) << ct << ":  ";
        // print full string:
        for (ulong k=0; k<n; ++k)  PRX();
        cout << "  " << setw(2) << j;  // period

        // print string in minimal period:
//        cout << "    ";
//        ulong k;
//        for (k=0; k<n-j; ++k)  cout << " ";
//        for (  ; k<n; ++k)  PRX();

        if ( P.is_necklace() ) cout << "  N";  // is a necklace
        if ( P.is_lyndon_word() ) cout << " L";  // is a Lyndon word
        cout << endl;

        jjassert( P.OK() );
#endif  // TIMING
    }
    while ( (j=P.next_pre()) );

    cout << "  # pre-necklaces = " << pct;
    cout << ",  # necklaces = " << ct;
    cout << ",  # Lyndon words = " << lct;
    cout << endl;

    return 0;
}
// -------------------------

/*
Timing:

Timing: (Intel(R) Core(TM) i7-8700K CPU @ 3.70GHz)

GCC 12.2


with NECKLACE_MAX_ARRAY_LEN defined:

time ./bin 12 8
arg 1: 12 == n  [length of strings >=1]  default=4
arg 2: 8 == m  [number of different beads >=2]  default=3
  # pre-necklaces = 6632235540  # necklaces = 5726645688  # Lyndon words = 5726600880
9.88user 0.00system 0:09.88elapsed 100%CPU
 ==> 5726645688 / 9.88 = 579,620,008 necklaces per second
*/


/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/necklace-demo.cc"
/// make-target2: "1demo DSRC=demo/comb/necklace-demo.cc DEMOFLAGS=-DTIMING"
/// End:

